import string

from django.http import HttpResponse
from django.contrib.auth.hashers import make_password
from main.models import DefaultSettings, Vm, User, Iso
from main.UserProxMoxDTO import UserProxMoxDTO
from main.VmProxMoxDTO import VmProxMoxDTO
from datetime import datetime, timedelta


def get_default_settings() -> dict:
    queryset = DefaultSettings.objects.all()
    output_settings_dict = {}
    for line in queryset:
        output_settings_dict['lease_time_months'] = line.lease_time_months
        output_settings_dict['memory_mbs'] = line.memory_mbs
        output_settings_dict['storage_gbs'] = line.storage_gbs
        output_settings_dict['cores'] = line.cores
        output_settings_dict['amount_of_vms'] = line.amount_of_vms
        output_settings_dict['notification_email'] = line.notification_email
    return output_settings_dict


def update_default_settings(lease_time: int, memory: int, storage: int, cores: int, amount_of_vms: int, notification_email: str) -> HttpResponse:
    ds = DefaultSettings.objects.get(id=1)
    ds.lease_time_months = lease_time
    ds.memory_mbs = memory
    ds.storage_gbs = storage
    ds.cores = cores
    ds.amount_of_vms = amount_of_vms
    ds.notification_email = notification_email
    ds.save()
    return HttpResponse("Updated successfully!")


# Returns id of last inserted VM
def insert_vm(vm: VmProxMoxDTO) -> int:
    vm_model = Vm()
    vm_model.name = vm.vm_name
    vm_model.memory_mbs = vm.vm_memory
    vm_model.storage_gbs = int(vm.vm_disk_size[:-1])
    vm_model.cores = vm.vm_cores
    vm_model.end_date = datetime.now() + timedelta(days=vm.vm_expire_days)
    results = Iso.objects.filter(volid=vm.vm_iso_file)
    if results.exists():
        vm_model.iso_id = results[0]
    else:
        vm_model.iso_id = None
    # vm_model.member_email
    vm_model.save()

    return vm_model.id


def add_user_to_vm(user_id: int, vm_id: int) -> HttpResponse:
    try:
        vm_model = Vm.objects.get(id=vm_id)
        print(vm_model.name)
    except Vm.DoesNotExist:
        # Handle the case when the object does not exist
        return HttpResponse("Object not found.")

    # If record of VM exists update it
    try:
        user = User.objects.get(user_id=user_id)
        print(user.email)
        vm_model.user_id = user
        vm_model.save()
    except User.DoesNotExist:
        # Handle the case when the object does not exist
        return HttpResponse("Object not found.")

    return HttpResponse("Record updated!")


def get_list_iso_from_db() -> tuple:
    queryset = Iso.objects.all()
    return queryset


def is_iso_in_sql_db(volidinp: str) -> bool:
    queryset = Iso.objects.filter(volid=volidinp)
    # If query set empty (no matches found) return false
    if queryset:
        return True
    else:
        return False


def insert_iso(os_name: str, volidinp: str) -> HttpResponse:
    iso_model = Iso()
    iso_model.os_name = os_name
    iso_model.volid = volidinp
    iso_model.is_visible = False
    iso_model.save()
    return HttpResponse("Data inserted!")


def remove_iso(id: int) -> HttpResponse:
    try:
        iso_model = Iso.objects.get(id=id)
        iso_model.delete()
    except Iso.DoesNotExist:
        # Handle the case when the object does not exist
        return HttpResponse("Object not found.")
    return HttpResponse("Data removed!")


# Returns a dictionary of volid - os_name of records, where is_visible = True
def get_list_visible_iso_from_db() -> dict:
    results = get_list_iso_from_db()
    output_iso = {}
    for record in results:
        if record.is_visible is True:
            output_iso[record.volid] = record.os_name
    return output_iso


def change_iso_visibility(id: int) -> HttpResponse:
    try:
        iso_model = Iso.objects.get(id=id)
        # Change is_visible field to the opposite
        if iso_model.is_visible == 1:
            iso_model.is_visible = 0
            iso_model.save()
        else:
            iso_model.is_visible = 1
            iso_model.save()
    except Iso.DoesNotExist:
        # Handle the case when the object does not exist
        return HttpResponse("Object not found.")
    return HttpResponse("Visbility Changed!")


def change_user_visibility(username: string) -> HttpResponse:
    try:
        user = User.objects.get(username=username)
        if user.is_active == 1:
            user.is_active = 0
            user.save()
        else:
            user.is_active = 1
            user.save()
    except User.DoesNotExist:
        return HttpResponse("User not found")
    return HttpResponse("Activity changed")


# Returns last inserted user's id
def insert_member(userdto: UserProxMoxDTO) -> int:
    limits = get_default_settings()
    user_model = User()
    user_model.email = userdto.email
    user_model.first_name = userdto.firstname
    user_model.last_name = userdto.lastname
    user_model.password = make_password(userdto.password)
    user_model.username = userdto.username
    user_model.maxAmountOfVms = limits['amount_of_vms']
    user_model.save()

    return user_model.user_id


def get_list_users() -> tuple:
    queryset = User.objects.all()
    return queryset


def get_list_unapproved_users() -> tuple:
    queryset = Iso.objects.filter(is_active=0)
    return queryset


def is_username_unique(username: str) -> bool:
    try:
        user_model = User.objects.get(username=username)
        if user_model is not None:
            return False
        else:
            return True
    except User.DoesNotExist:
        return True


def get_user_vm_count(user_inp: User) -> int:
    queryset = Vm.objects.filter(user_id=user_inp)
    return queryset.count()


def edit_iso_db(iso_id: int, os_name_inp: str) -> HttpResponse:
    try:
        iso_model = Iso.objects.get(id=iso_id)
        iso_model.os_name = os_name_inp
        iso_model.save()
    except Iso.DoesNotExist:
        return HttpResponse("Object not found.")
    return HttpResponse("Changes saved!")


def get_iso_by_id(iso_id: int) -> dict:
    result = Iso.objects.get(id=iso_id)
    iso_dict = {
        'os_name': result.os_name,
        'iso_id': result.id,
        'volid': result.volid,
        'is_visible': result.is_visible
    }
    return iso_dict


def get_user_info_from_vmid(vmid: int):
    try:
        # Connect to the database and retrieve the VM object using vmid
        vm = Vm.objects.get(id=vmid)
        # Retrieve user_id and user_name from the VM object
        user_temp = vm.user_id
        user_name = str(user_temp.first_name) + " " + str(user_temp.last_name)
        end_date = vm.end_date

        return user_temp.user_id, user_name, end_date

    except Vm.DoesNotExist:
        # Handle the case when VM is not found in the database
        return None, None, None

    except Exception as e:
        # Handle any other exceptions that may occur
        print(f"An error occurred: {e}")
        return None, None, None


def remove_vm_db(vmid: int):
    try:
        vm_model = Vm.objects.get(id=vmid)
        vm_model.delete()
    except Iso.DoesNotExist:
        # Handle the case when the object does not exist
        return HttpResponse("Object not found.")
    return HttpResponse("Data removed!")




