from django.contrib.admin import display
from django.db import models
from django.contrib.auth.models import AbstractUser


# Create your models here.
# REQUIRED_FIELDS = ['id_email', 'first_name', 'last_name']


class User(AbstractUser):
    user_id = models.AutoField(primary_key=True, unique=True)
    email = models.EmailField(max_length=255)
    first_name = models.CharField(max_length=128, unique=False, null=True)
    last_name = models.CharField(max_length=128, unique=False, null=True)
    is_active = models.BooleanField(default=False)
    is_staff = models.BooleanField(default=False)
    maxAmountOfVms = models.IntegerField(null=True)
    is_superuser = models.BooleanField(default=False)
    password = models.CharField(max_length=255)
    vlanTag = models.IntegerField(null=True)
    last_login = models.DateTimeField(auto_now=True)
    date_joined = models.DateTimeField(auto_now_add=True)
    #
    # def __str__(self):
    #     return self.username, self.first_name, self.last_name


class DefaultSettings(models.Model):
    id = models.AutoField(primary_key=True, unique=True)
    lease_time_months = models.IntegerField()
    memory_mbs = models.IntegerField()
    storage_gbs = models.IntegerField()
    cores = models.IntegerField()
    notification_email = models.EmailField()
    amount_of_vms = models.IntegerField()

    # REQUIRED_FIELDS = ['id', 'lease_time_months', 'memory_mbs', 'storage_gbs', 'cores', 'notification_email', 'amount_of_vms']

    # def __str__(self):
    #     return self.id, self.lease_time_months, self.memory_mbs, self.storage_gbs, self.cores, self.notification_email, self.amount_of_vms


class Iso(models.Model):
    id = models.AutoField(primary_key=True, unique=True)
    is_visible = models.BooleanField()
    os_name = models.CharField(max_length=255)
    volid = models.CharField(max_length=320)

    # REQUIRED_FIELDS = ['id', 'hashed_file', 'is_visible', 'os_name']
    # #
    # # def __str__(self):
    # #     return self.id, self.hashed_file, self.is_visible, self.os_name


class Vm(models.Model):
    id = models.AutoField(primary_key=True, unique=True)
    name = models.CharField(max_length=64)
    user_id = models.ForeignKey(User, on_delete=models.CASCADE, null=True)
    end_date = models.DateTimeField()
    memory_mbs = models.IntegerField()
    storage_gbs = models.IntegerField()
    cores = models.IntegerField()
    iso_id = models.ForeignKey(Iso, on_delete=models.SET_NULL, max_length=11, null=True)

    # REQUIRED_FIELDS = ['id', 'name', 'memory_mbs', 'storage_gbs', 'cores']  # NOT NULL fields
    #
    # def __str__(self):
    #     return self.id, self.name, self.memory_mbs, self.storage_gbs, self.cores


class RequestType(models.Model):
    id = models.AutoField(primary_key=True, unique=True)
    name = models.CharField(max_length=20, unique=True)

    # REQUIRED_FIELDS = ['id', 'name']
    #
    # def __str__(self):
    #     return self.id, self.name


class Request(models.Model):
    id = models.AutoField(primary_key=True, unique=True)
    message = models.CharField(max_length=256)
    vm_id = models.ForeignKey(Vm, on_delete=models.CASCADE, max_length=11)
    request_type_id = models.ForeignKey(RequestType, on_delete=models.SET_NULL, max_length=11, null=True)
    is_approved = models.BooleanField()
    date_time = models.DateTimeField(auto_now=True)
    user_id = models.ForeignKey(User, on_delete=models.CASCADE)

    # REQUIRED_FIELDS = ['id', 'request_type_id', 'date_time']
    #
    # def __str__(self):
    #     return self.id, self.request_type_id, self.date_time
