import time

from main.UserProxMoxDTO import UserProxMoxDTO
import os
from django.shortcuts import render
from django.urls import reverse
from django.contrib import messages
from main.Utils import create_vm, get_list_iso_urls, sanitize_vm_data_input, months_to_days, upload_iso, \
    sync_iso_from_ve_to_db, remove_iso_from_ve, create_user, is_email_nhl, change_user_enable, create_vm_from_template, \
    superuser_check, get_list_templates
from main.db_functions import get_default_settings, insert_vm, insert_iso, get_list_visible_iso_from_db, remove_iso, \
    change_iso_visibility, insert_member, is_username_unique, change_user_visibility, add_user_to_vm, get_user_vm_count, \
    edit_iso_db, get_iso_by_id, update_default_settings, get_user_info_from_vmid, remove_vm_db
from main import default_vm_node, default_storage, default_iso_storage, default_iso_path
from django.http import HttpResponse, HttpResponseRedirect
from main.VmProxMoxDTO import VmProxMoxDTO
from django.shortcuts import redirect
from main.models import Iso, User
from proxmox.local_settings import MEDIA_ROOT
from django.contrib.auth.decorators import login_required, user_passes_test
from proxmox.settings import LOGIN_REDIRECT_URL
from main import proxmoxAuth
from main.models import Vm
import proxmox



@login_required
def create_vm_view(request):
    # Get a list of limits from mysql DB
    limits = get_default_settings()
    user_logged = request.user  # Logged in user info (Django model)
    user_vm_count = get_user_vm_count(user_logged)
    # When user submits the form
    if request.POST:
        vm_name = request.POST.get('vm_name') + 'User' + user_logged.username  # Name for VM - VmNameUserUserName
        vm_cores = int(request.POST.get('vm_cores'))  # Number of CPU cores
        vm_memory = int(request.POST.get('vm_memory'))  # Memory size in MB
        vm_disk_size = int(request.POST.get('disk_size'))  # Disk size for the VM
        iso_file = str(request.POST.get('iso'))  # Path to the ISO file
        vmTempDTO = VmProxMoxDTO(vm_name, vm_cores, vm_memory, vm_disk_size,
                                 months_to_days(int(limits['lease_time_months'])), iso_file)
        vm_sanitized = sanitize_vm_data_input(vmTempDTO)

        # Check user's VM count
        if user_vm_count >= int(user_logged.maxAmountOfVms):
            # Show an alert message
            messages.info(request, "You have exceeded your limit of available VMs")
            return render(request, "main/create.html")
        try:
            last_vm_id = insert_vm(vm_sanitized)
            info = create_vm(vm_sanitized, last_vm_id)
            add_user_to_vm(user_logged.user_id, last_vm_id)
            # Redirect user to a different page if vm created
            url = reverse('create_vm_view')
            return HttpResponseRedirect(url)
        except Exception as error:
            return HttpResponse(error)

    # Get a list of all ISO files from proxmox
    results = get_list_visible_iso_from_db()

    context = {
        "iso_options": results,
        "disk_size": limits['storage_gbs'],
        "memory": limits['memory_mbs'],
        "cores": limits['cores'],
        "lease": limits['lease_time_months'],
        "vm_count": user_vm_count
    }
    return render(request, "main/create.html", context)


def redirect_home(request):
    return render(request, "main/home.html")


@login_required
def index(request):
    username = request.user.username
    return HttpResponse("Hello " + username)


@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def upload_iso_view(request):
    if request.method == 'POST':
        iso_file = request.FILES['iso_file']  # Get ISO file from web form
        os_name = request.POST.get('os_name')  # Get OS name
        upload_path = os.path.join(MEDIA_ROOT, iso_file.name)
        try:
            # Save the uploaded file to the upload path
            with open(upload_path, 'wb+') as destination:
                for chunk in iso_file.chunks():
                    destination.write(chunk)
            # Upload the ISO file to Proxmox VE
            volid = upload_iso(iso_file.name, upload_path)
        except Exception as error:
            return HttpResponse(error)
        # Save to sql db
        if len(os_name) == 0:
            os_name = volid
        insert_iso(os_name, volid)
        url = reverse('manage_iso')
        return HttpResponseRedirect(url)
    return render(request, 'admin/upload_iso.html')  # Render the upload form initially


@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def dynamic_iso_table(request):
    entries = Iso.objects.all()
    return render(request, 'admin/manage_iso.html', {'entries': entries})


@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def dynamic_user_table(request):
    # gets all the inactive users
    entries = User.objects.filter(is_active=0)
    return render(request, 'admin/user_register_requests.html', {'entries': entries})


@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def synchronize_iso_view(request):
    sync_iso_from_ve_to_db()
    url = reverse('manage_iso')
    return HttpResponseRedirect(url)


@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def delete_iso(request, id=None):
    # Get data from Get request if its not empty
    iso_id = id
    if iso_id is not None:
        response_ve = remove_iso_from_ve(iso_id)  # Remove from Ve
        response_db = remove_iso(iso_id)  # Remove from DB
    url = reverse('manage_iso')
    return HttpResponseRedirect(url)


@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def change_iso_visibility_view(request, id=None):
    # Get data from post
    iso_id = id
    if iso_id is not None:
        response = change_iso_visibility(iso_id)
    url = reverse('manage_iso')
    return HttpResponseRedirect(url)


@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def edit_iso_view(request):
    iso_id = request.session.get('iso_edit_id')
    # Get ISO info from DB
    iso_dict = get_iso_by_id(iso_id)
    # When user submits form -> save changes and redirect back
    if request.method == 'POST':
        if iso_id is not None:
            os_name = str(request.POST.get('os_name'))
            if len(os_name) == 0:
                os_name = iso_dict['os_name']
            response = edit_iso_db(iso_id, os_name)
            return HttpResponseRedirect(reverse('manage_iso'))

    return render(request, 'admin/edit_iso.html', {'entries': iso_dict})


@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def edit_iso_redirect_view(request, id=None):
    user = request.user
    if user.is_superuser is False:
        url = reverse('admin_login')
        return HttpResponseRedirect(url)

    if id is not None:
        request.session['iso_edit_id'] = id  # Save ISO id into session variable
        return HttpResponseRedirect(reverse('edit_iso'))

    # Go back if there is an issue with id in GET request
    url = reverse('manage_iso')
    return HttpResponseRedirect(url)


@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def change_user_activity(request, username=None):
    user_name = username
    if user_name is not None:
        response = change_user_visibility(user_name)
        response_ve = change_user_enable(user_name)
    url = reverse('manage_user')
    return HttpResponseRedirect(url)


def create_user_view(request):
    # When user submits form
    if request.POST:
        # Get all input info into a DTO
        email = request.POST.get('email')
        first_name = request.POST.get('fname')
        last_name = request.POST.get('lname')
        password = str(request.POST.get('password'))
        username = request.POST.get('username')

        # Check if email is from NHL
        if not is_email_nhl(email):
            messages.info(request, "Only @nhlstenden.com email address can be used")
            return render(request, "main/create_user.html")
        # Check if username is unique
        if not is_username_unique(username):
            messages.info(request, "Username already exists")
            return render(request, "main/create_user.html")

        userdto = UserProxMoxDTO(email, first_name, last_name, password, username)
        try:
            user_id = insert_member(userdto)  # Save to db
            response = create_user(userdto, user_id)  # Save to proxmox
            print(response)
        except Exception as error:
            return HttpResponse(error)
    return render(request, "main/create_user.html")


@login_required
def templates_list_result(request):
    templates = get_list_templates()
    limits = get_default_settings()
    user_logged = request.user  # Logged in user info (Django model)
    user_vm_number = get_user_vm_count(user_logged)
    context = {
        "templates": templates,
        "disk_size": limits['storage_gbs'],
        "memory": limits['memory_mbs'],
        "cores": limits['cores'],
        "lease": limits['lease_time_months'],
        "vm_count": user_vm_number
    }
    if request.POST:
        # Check user's VM count
        if user_vm_number >= int(limits['amount_of_vms']):
            # Show an alert message
            messages.info(request, "You have exceeded your limit of available VMs")
            return render(request, "main/create_template.html", context)
        template_id = request.POST.get('template')
        vm_name = request.POST.get('vm_name') + 'User' + user_logged.username  # Name for VM - VmNameUserUserName
        vm_cores = limits['cores']
        vm_memory = limits['memory_mbs']
        vm_disk_size = limits['storage_gbs']
        iso_file = None
        vmTempDTO = VmProxMoxDTO(vm_name, vm_cores, vm_memory, vm_disk_size, months_to_days(int(limits['lease_time_months'])), iso_file)
        vm_sanitized = sanitize_vm_data_input(vmTempDTO)
        last_vm_id = insert_vm(vm_sanitized)  # Save to db
        info = create_vm_from_template(last_vm_id, vm_name, template_id)
        add_user_to_vm(user_logged.user_id, last_vm_id)
        # Redirect user to a different page if vm created
        return HttpResponseRedirect(reverse('templates'))

    return render(request, "main/create_template.html", context)


@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def default_settings_view(request):
    if request.POST:
        lease = int(request.POST.get('lease_time'))
        memory = int(request.POST.get("memory"))
        storage = int(request.POST.get("storage"))
        cores = int(request.POST.get("cores"))
        vm_amount = int(request.POST.get("amount_of_vms"))
        email = request.POST.get("notification_email")

        result = update_default_settings(lease, memory, storage, cores, vm_amount, email)

    limits = get_default_settings()
    context = {
        "disk_size": limits['storage_gbs'],
        "memory": limits['memory_mbs'],
        "cores": limits['cores'],
        "lease": limits['lease_time_months'],
        "vm_amount": limits['amount_of_vms'],
        "notification_email": limits['notification_email']
    }
    return render(request, 'admin/manage_default_settings.html', context)


@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def delete_vm(request):
    if request.method == 'POST':
        # Get the VM ID from the submitted form
        vm_id = request.POST['vm_id']

        try:
            proxmoxAuth.nodes(default_vm_node).qemu(vm_id).post('status/stop')  # Stop vm if its running
            time.sleep(1) # Wait for vm to stop

            proxmoxAuth.nodes(default_vm_node).qemu(vm_id).delete()  # Delete from proxmox ve
            remove_vm_db(vm_id)  # Delete from DB
            return redirect('vms')  # Redirect to the vms page
        except Exception as e:
            return HttpResponse({'error': str(e)})

    return render(request, 'admin/delete.html')


@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def view_vm(request):
    vms = proxmoxAuth.cluster.resources.get(type='vm')

    vm_list = []
    for vm in vms:
        vmid = vm['vmid']
        user_id, user_name, end_date = get_user_info_from_vmid(vmid)  # Call the function to get user_id and user_name

        vm_list.append({
            'status': vm['status'],
            'vmid': vmid,
            'user_id': user_id,
            'user_name': user_name,
            'end_date': end_date

        })

    return render(request, 'admin/delete.html', {'vms': vm_list})


