"""
URL configuration for proxmox project.

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/4.2/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
from django.contrib import admin
from django.urls import path, include
from main import views as views
from main import contact

admin.site.site_header = "Proxmox Self Service"
admin.site.index_title = "Welcome Admin"

app_name = "proxmox"
urlpatterns = [
    path("", views.index, name="index"),
    path("admin/", admin.site.urls, name="admin_login"),
    path("proxmox/", views.create_vm_view, name="create_vm_view"),
    path("home/", views.redirect_home, name="redirect_home"),
    path("upload_iso/", views.upload_iso_view, name="upload_iso"),
    path("manage_iso/", views.dynamic_iso_table, name="manage_iso"),
    path("sync_iso/", views.synchronize_iso_view, name="sync_iso"),
    path("delete_iso/<int:id>", views.delete_iso, name="delete_iso"),
    path("change_iso/<int:id>", views.change_iso_visibility_view, name="change_iso"),
    path("create_user/", views.create_user_view, name="create_user"),
    path('accounts/', include('django.contrib.auth.urls'), name="login_user"),
    path("manage_user/", views.dynamic_user_table, name="manage_user"),
    path("change_user/<str:username>", views.change_user_activity, name="change_user_activity"),
    path("edit_iso_redirect/<int:id>", views.edit_iso_redirect_view, name="edit_iso_redirect"),
    path("edit_iso", views.edit_iso_view, name="edit_iso"),
    path("templates/", views.templates_list_result, name="templates"),
    path("default_settings/", views.default_settings_view, name="change_default_settings"),
    path('vms/', views.delete_vm, name='delete'),
    path('vms/view/', views.view_vm, name='vms'),
    path('send_email/', contact.send_email, name='send_email')

]
